package pl.soppo.ucwizard.service 
{
	import flash.desktop.NativeProcess;
	import flash.desktop.NativeProcessStartupInfo;
	import flash.display.Loader;
	import flash.events.Event;
	import flash.events.NativeProcessExitEvent;
	import flash.events.ProgressEvent;
	import flash.filesystem.File;
	import flash.filesystem.FileMode;
	import flash.filesystem.FileStream;
	import flash.utils.ByteArray;
	import org.robotlegs.mvcs.Actor;
	import pl.soppo.ucwizard.controler.events.GenerateApplicationEvent;
	import pl.soppo.ucwizard.controler.events.ProgressGenerationEvent;
	import pl.soppo.ucwizard.model.data.Globals;
	import pl.soppo.ucwizard.model.data.IconData;
	import pl.soppo.ucwizard.model.data.ProjectData;
	import pl.soppo.ucwizard.model.ProjectModel;
	import pl.soppo.ucwizard.service.parsers.AppConfParser;
	
	/**
	 * ...
	 * @author Maciej Reichwald
	 */
	public class ApplicationGeneratorService extends Actor 
	{
		[Inject]
		public var appConf:AppConfParser;
		
		[Inject]
		public var model:ProjectModel;
		
		private var path:String = "";
		private var progressTotal:Number;
		private var _currentProgress:Number;
		private var isValid:Boolean = true;
		
		public function generate():void
		{
			path = (model.projectList[model.currentProject] as ProjectData).generate.destinationFolder.url;
			
			setCurrentProgress(0);
			countProgressTotal();
			
			createFile(
				"app.conf", 
				appConf.parseAppConf(),
				path + "\\data");
				
			createFile(
				"local.conf", 
				appConf.parseLocalConf(),
				path + "\\data");
				
			createFile(
				"icons.conf", 
				appConf.parseIconsConf(),
				path + "\\data");
			
			if( (model.projectList[model.currentProject] as ProjectData).graphics.skin != null )
				copyFile(
					"\\skin.swf", 
					(model.projectList[model.currentProject] as ProjectData).graphics.skin.url,
					"\\data");
			else setCurrentProgress(_currentProgress + 1);;
			
			if((model.projectList[model.currentProject] as ProjectData).course.chapters.children().length() > 0)
			{
				createFile(
					(model.projectList[model.currentProject] as ProjectData).general.name+".cs",
					(model.projectList[model.currentProject] as ProjectData).course.chapters,
					path + "\\data\\content");
			}
			else 
			{
				dispatch(new GenerateApplicationEvent(GenerateApplicationEvent.GENERATE_APP_RESULT, "Error!\n\nNo lessons added."));
				isValid = false;
			}
				
			createFile(
				"content_to_name.xml",
				appConf.parseContentToName(),
				path + "\\data\\content");
				
			createJarFile(
				"keyWordsSearcher.jar", 
				new Globals.KeyWordsSearcher() as ByteArray, 
				File.applicationStorageDirectory.nativePath);
				
			/*createFile(
				"generateKeyWordsXML.bat",
				appConf.parseGenerateKeyWordsXML(),
				File.applicationStorageDirectory.nativePath);*/
			
			launchFile(
				"java.exe", 
				"C:/Program Files (x86)/Java/jre6/bin",
				appConf.parseGenerateKeyWordsXML());	
				
			copyIcons((model.projectList[model.currentProject] as ProjectData).course.icons, "\\data\\assets\\icons");
			copyIcons((model.projectList[model.currentProject] as ProjectData).graphics.icons, "\\Others\\autorun");
				
			copyCSFiles();	
				
			/*var args:Vector.<String> = new Vector.<String>()
			args.push(File.applicationStorageDirectory.nativePath + "\\generateKeyWordsXML.bat");
			launchFile(
				"/calc.exe", 
				"C:/Windows/System32",
				null);*/
		}
		
		private function countProgressTotal():void
		{
			progressTotal = 8;
			progressTotal += (model.projectList[model.currentProject] as ProjectData).course.icons.length;
			progressTotal += (model.projectList[model.currentProject] as ProjectData).graphics.icons.length;
			
			var chapters:XMLList = (model.projectList[model.currentProject] as ProjectData).course.chapters;
			var files:XMLList = chapters.descendants().(name().localName == "item");
			progressTotal += files.length();
			
		}
		
		private function copyIcons(icons:Vector.<IconData>, destiny:String):void
		{
			if ( !isValid) return;
			
			for (var i:int = 0; i < icons.length; i++)
				copyFile(
					"\\"+icons[i].name,
					icons[i].url,
					destiny,
					false
					);		
		}
		
		private function copyCSFiles():void
		{
			if ( !isValid) return;
			
			var chapters:XMLList = (model.projectList[model.currentProject] as ProjectData).course.chapters;
			var files:XMLList = chapters.descendants().(name().localName == "item");

			for (var i:int = 0; i < files.length(); i++)
			{
				var source:String = (model.projectList[model.currentProject] as ProjectData).course.source;
					var filePath:String = source.substr(0, source.lastIndexOf("\\")) + "\\";
					var fileHref:String = files[i].attributes().(name().localName == "href");
					filePath += fileHref.replace("/", "\\");
					
					var file:File = new File(filePath);
					var destPath:String = path + "\\data\\content\\" + fileHref;
					destPath = destPath.substr(0,destPath.lastIndexOf("/"));
					
					var dest:File = File.desktopDirectory.resolvePath(destPath);
					dest.createDirectory();
					
					var fileDestPath:String = dest.nativePath + "\\" + file.name;
					var fileDest:File = File.desktopDirectory.resolvePath(fileDestPath);
					
					if ( file.exists ) file.copyTo(fileDest, true);
					
					var dirPath:String = filePath.substr(0, filePath.lastIndexOf("."));
					var dir:File = File.desktopDirectory.resolvePath(dirPath);
					
					if ( dir.exists )
					{
						var dirDestPath:String = path + "\\data\\content\\"+fileHref;
						dirDestPath = dirDestPath.substr(0,dirDestPath.lastIndexOf("."));
				
						var dirDest:File = File.desktopDirectory.resolvePath(dirDestPath);
						dirDest.createDirectory();
						
						dir.addEventListener(Event.COMPLETE, handlePartComplete);
						dir.copyToAsync(dirDest, true);
					}
					else 
					{
						dispatch(new GenerateApplicationEvent(GenerateApplicationEvent.GENERATE_APP_RESULT, "Error!\n\nMissing " + dir.name + " folder."));
						isValid = false;
						break;
					}
			}
				
		}
		
		private function launchFile(name:String, filePath:String, fileArguments:Vector.<String>):void
		{
			if ( !isValid) return;
			
			if ( NativeProcess.isSupported )
			{
				try 
				{
					var file:File = File.applicationDirectory.resolvePath(filePath + "/" + name);
					
					var nativeProcess:NativeProcessStartupInfo = new NativeProcessStartupInfo();
					nativeProcess.executable = file;
					nativeProcess.workingDirectory = File.applicationDirectory;
					nativeProcess.arguments = fileArguments;
					
					if(fileArguments != null)
					for (var i:int = 0; i < fileArguments.length; i++)
						trace(fileArguments[i]);
					
					var process:NativeProcess = new NativeProcess();
					process.addEventListener( ProgressEvent.STANDARD_OUTPUT_DATA, handleOutpur);
					process.addEventListener( ProgressEvent.STANDARD_ERROR_DATA, handleError);
					process.addEventListener( NativeProcessExitEvent.EXIT, handlePartComplete);
					process.start(nativeProcess);
				} 
				catch (e:Error)
				{
					dispatch(new GenerateApplicationEvent(
						GenerateApplicationEvent.GENERATE_APP_RESULT, 
						"Error!\n\nError during genereting XML search files.\n" +e
				));
					isValid = false;
				}
			}
			else
			{
				dispatch(new GenerateApplicationEvent(GenerateApplicationEvent.GENERATE_APP_RESULT, "Error!\n\nNative process not supported.\nUpdate you AIR version to 2.6 or newer."));
				isValid = false;
				//setCurrentProgress(_currentProgress + 1);
			}
		}
		
		private function handlePartComplete(e:Event):void 
		{
			if(e.currentTarget as File != null)
				setCurrentProgress(_currentProgress + 1, "Coping " + (e.currentTarget as File).name+"...");
			else if(e.currentTarget as NativeProcess != null)
				setCurrentProgress(_currentProgress + 1, "Generating XML search files...");
			else setCurrentProgress(_currentProgress + 1);
		}
		
		private function handleError(e:ProgressEvent):void 
		{
			dispatch(new GenerateApplicationEvent(
				GenerateApplicationEvent.GENERATE_APP_RESULT, 
				"Error!\n\nError during genereting XML search files.\n" +
				(e.currentTarget as NativeProcess).standardError.readUTFBytes((e.currentTarget as NativeProcess).standardError.bytesAvailable)
				));
			isValid = false;
		}
		
		private function handleOutpur(e:ProgressEvent):void 
		{
			trace((e.currentTarget as NativeProcess).standardOutput.readUTFBytes((e.currentTarget as NativeProcess).standardOutput.bytesAvailable));
		}
		
		private function createJarFile(name:String, data:ByteArray, path:String):void
		{
			if ( !isValid) return;
			
			var file:File = File.applicationDirectory.resolvePath(path + "\\" + name);
			if ( !file.exists )
			{
				var fileStream:FileStream = new FileStream();
				fileStream.open(file, FileMode.WRITE);
				fileStream.writeBytes(data, 0, data.bytesAvailable);
				fileStream.close();
			}
			
			setCurrentProgress(_currentProgress + 1, "Creating jar lib...");
		}
		
		private function createFile(fileName:String, fileContent:String, filePath:String=""):void
		{
			if ( !isValid) return;
			
			if ( filePath == "" ) filePath = path;
			else createDirectory(filePath);
			
			var file:File = new File(filePath+"\\"+fileName);
			var fileStream:FileStream = new FileStream();
			fileStream.open(file, FileMode.WRITE);
			
			fileStream.writeUTFBytes(fileContent);
			fileStream.close();
			
			setCurrentProgress(_currentProgress + 1, "Creating "+fileName+"...");
		}
		
		private function createDirectory(dirpath:String):void
		{
			var directory:File = File.desktopDirectory;
			directory.resolvePath(dirpath);
			directory.createDirectory();
		}
		
		private function copyFile(name:String, filePath:String, fileDir:String="", copyIfExists:Boolean=true):void
		{
			if ( !isValid) return;
			
			var skinFile:File = File.desktopDirectory.resolvePath(filePath);

			if ( fileDir != "")
				createDirectory(path+fileDir);
			
			var newSkin:File = File.desktopDirectory.resolvePath(path + fileDir + name);

			if (skinFile.exists)
			{
				if ( !newSkin.exists || copyIfExists )
				{
					skinFile.addEventListener(Event.COMPLETE, handleCopyComplete);
					skinFile.copyToAsync(newSkin, true);
				}
				else setCurrentProgress(_currentProgress + 1, "Copying "+skinFile.name+"...");
			}
			else
			{
				dispatch(new GenerateApplicationEvent(GenerateApplicationEvent.GENERATE_APP_RESULT, "Error!\n\nFile " + skinFile.name + " does not exit."));
				isValid = false;
			}
		}
		
		private function handleCopyComplete(e:Event):void 
		{
			setCurrentProgress(_currentProgress + 1, "Copying "+(e.currentTarget as File).name+"...");
		}
		
		private function setCurrentProgress(value:Number, label:String=""):void 
		{
			_currentProgress = value;
			if ( _currentProgress / progressTotal == 1 && isValid) 
			{
				label = " ";
				dispatch(new GenerateApplicationEvent(GenerateApplicationEvent.GENERATE_APP_RESULT, "Success!\n\nApplication created."));
			}
			dispatch( new ProgressGenerationEvent(ProgressGenerationEvent.GENERATION_PROGRESS, _currentProgress / progressTotal, label));
			
		}
		
		
	}

}